package ua.in.hutorny.fsa.primer;

import ua.in.hutorny.fsa.model.*;

/**
 * State Machine of a simple microwave oven (MW) with one push button
 */
public class MicrowaveStateMachine {
  
  interface OnDoorOpen extends Event {}
  interface OnDoorClose extends Event {}
  interface OnPushButton extends Event {}
  interface OnCookTimerExpired extends Event {}
  interface IfTimerIsSet extends Condition {} 
  interface IfTimerIsNotSet extends Condition {}
  interface DoDing extends Action {}
  interface DoDong extends Action {}
  interface DoStartHeater extends Action {}
  interface DoStopHeater extends Action {}
  interface DoBeep extends Action {}
  interface DoStartCookTimer extends Action {}
  interface DoSuspendCookTimer extends Action {}
 
  /**
   * In Door Open State MW waits until the door is closed
   */
  interface DoorOpenState extends State {
	  @OnEnter void enter	(DoDing a);                            	// on entering Door Open State issue a Ding action
	  WaitingState  idle   	(OnDoorClose e, IfTimerIsSet a);  		// on Door Close Event transit to Waiting State 
	  HeatingState  resume 	(OnDoorClose e, IfTimerIsNotSet a); 	// on Door Close Event transit to Waiting State 
	  DoorOpenState ignore 	(OnPushButton e, DoBeep a);          	// on Push Button Event issue a Beep and remain in DoorOpenState 
	  @OnLeave void leave	(DoDong a);								// on entering Door Close State issue a Dong  
  }
  /**
   * In Waiting State MW waits until a button is pushed
   * If timer is set, it starts heating, otherwise it beeps
   */
  interface WaitingState extends State {
	  HeatingState  start   (OnPushButton e, IfTimerIsSet c);		// on Push Button Event if timer is set transit to heating
	  WaitingState  beep    (OnPushButton e, IfTimerIsNotSet c, DoBeep a); // otherwise issue a Beep and remain in waiting
	  DoorOpenState open    (OnDoorOpen e);							// on Door Open Event transit to Door Open State 
  }
  /**
   * In Heating state it actually heats. 
   * On entering this state MW starts the heater and the cook timer; 
   * On leaving this state MW stops the heater and suspends the cook timer    
   */
  interface HeatingState extends State {
	  @OnEnter void enter	(DoStartHeater a, DoStartCookTimer t); 	// on entering HeatingState State issue Start Heater Action 
	  WaitingState  timer	(OnCookTimerExpired e);					// on Timer Expired Event transit to Waiting State  
	  WaitingState  stop	(OnPushButton e);						// on Push Button Event transit to WaitingState State   
	  DoorOpenState open	(OnDoorOpen e);							// on Door Open Event transit to Door Open State  
	  @OnLeave void leave	(DoStopHeater a, DoSuspendCookTimer b);	// on leave issue DoStopHeater  
  }
}
